@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2024 Advanced Micro Devices, Inc.
+ * Copyright (C) 2024-2025 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -110,6 +110,8 @@ typedef rocsparseio_enum_t rocsparseio_index_base_t;
 #define ROCSPARSEIO_TYPE_COMPLEX32 4
 #define ROCSPARSEIO_TYPE_COMPLEX64 5
 #define ROCSPARSEIO_TYPE_INT8 6
+#define ROCSPARSEIO_TYPE_FLOAT16 7
+#define ROCSPARSEIO_TYPE_BFLOAT16 8
 
 #define ROCSPARSEIO_FORMAT_DENSE_VECTOR 0
 #define ROCSPARSEIO_FORMAT_DENSE_MATRIX 1
@@ -137,6 +139,8 @@ typedef enum rocsparseio_type_
 {
     rocsparseio_type_int32     = ROCSPARSEIO_TYPE_INT32,
     rocsparseio_type_int64     = ROCSPARSEIO_TYPE_INT64,
+    rocsparseio_type_float16   = ROCSPARSEIO_TYPE_FLOAT16,
+    rocsparseio_type_bfloat16  = ROCSPARSEIO_TYPE_BFLOAT16,
     rocsparseio_type_float32   = ROCSPARSEIO_TYPE_FLOAT32,
     rocsparseio_type_float64   = ROCSPARSEIO_TYPE_FLOAT64,
     rocsparseio_type_complex32 = ROCSPARSEIO_TYPE_COMPLEX32,
