@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2023 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -60,7 +60,7 @@ void testing_bsrmv_bad_arg(const Arguments& arg)
 #define PARAMS_ANALYSIS \
     handle, dir, trans, mb, nb, nnzb, descr, bsr_val, bsr_row_ptr, bsr_col_ind, block_dim, info
 
-    auto_testing_bad_arg(rocsparse_bsrmv_ex_analysis<T>, PARAMS_ANALYSIS);
+    bad_arg_analysis(rocsparse_bsrmv_analysis<T>, PARAMS_ANALYSIS);
 
 #define PARAMS                                                                                     \
     handle, dir, trans, mb, nb, nnzb, alpha_device_host, descr, bsr_val, bsr_row_ptr, bsr_col_ind, \
@@ -69,12 +69,11 @@ void testing_bsrmv_bad_arg(const Arguments& arg)
     {
         static constexpr int num_exclusions  = 1;
         static constexpr int exclude_args[1] = {12};
-        auto_testing_bad_arg(rocsparse_bsrmv_ex<T>, num_exclusions, exclude_args, PARAMS);
+        select_bad_arg_analysis(rocsparse_bsrmv<T>, num_exclusions, exclude_args, PARAMS);
     }
 
-    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrmv_ex_clear(nullptr, info),
-                            rocsparse_status_invalid_handle);
-    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrmv_ex_clear(handle, nullptr),
+    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrmv_clear(nullptr, info), rocsparse_status_invalid_handle);
+    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrmv_clear(handle, nullptr),
                             rocsparse_status_invalid_pointer);
 
     for(auto matrix_type : rocsparse_matrix_type_t::values)
@@ -82,10 +81,9 @@ void testing_bsrmv_bad_arg(const Arguments& arg)
         if(matrix_type != rocsparse_matrix_type_general)
         {
             CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_type(descr, matrix_type));
-            EXPECT_ROCSPARSE_STATUS(rocsparse_bsrmv_ex_analysis<T>(PARAMS_ANALYSIS),
-                                    rocsparse_status_not_implemented);
-            EXPECT_ROCSPARSE_STATUS(rocsparse_bsrmv_ex<T>(PARAMS),
+            EXPECT_ROCSPARSE_STATUS(rocsparse_bsrmv_analysis<T>(PARAMS_ANALYSIS),
                                     rocsparse_status_not_implemented);
+            EXPECT_ROCSPARSE_STATUS(rocsparse_bsrmv<T>(PARAMS), rocsparse_status_not_implemented);
         }
     }
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_type(descr, rocsparse_matrix_type_general));
@@ -95,63 +93,74 @@ void testing_bsrmv_bad_arg(const Arguments& arg)
         if(operation != rocsparse_operation_none)
         {
             trans = operation;
-            EXPECT_ROCSPARSE_STATUS(rocsparse_bsrmv_ex_analysis<T>(PARAMS_ANALYSIS),
-                                    rocsparse_status_not_implemented);
-            EXPECT_ROCSPARSE_STATUS(rocsparse_bsrmv_ex<T>(PARAMS),
+            EXPECT_ROCSPARSE_STATUS(rocsparse_bsrmv_analysis<T>(PARAMS_ANALYSIS),
                                     rocsparse_status_not_implemented);
+            EXPECT_ROCSPARSE_STATUS(rocsparse_bsrmv<T>(PARAMS), rocsparse_status_not_implemented);
         }
     }
     trans = rocsparse_operation_none;
 
+    // block_dim == 0
+    block_dim = 0;
+    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrmv_analysis<T>(PARAMS_ANALYSIS),
+                            rocsparse_status_invalid_size);
+    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrmv<T>(PARAMS), rocsparse_status_invalid_size);
+    block_dim = safe_size;
+
 #undef PARAMS_ANALYSIS
 #undef PARAMS
 
     // Additional tests for invalid zero matrices
-    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrmv_ex_analysis<T>(handle,
-                                                           dir,
-                                                           trans,
-                                                           mb,
-                                                           nb,
-                                                           nnzb,
-                                                           descr,
-                                                           nullptr,
-                                                           bsr_row_ptr,
-                                                           nullptr,
-                                                           block_dim,
-                                                           info),
+    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrmv_analysis<T>(handle,
+                                                        dir,
+                                                        trans,
+                                                        mb,
+                                                        nb,
+                                                        nnzb,
+                                                        descr,
+                                                        nullptr,
+                                                        bsr_row_ptr,
+                                                        nullptr,
+                                                        block_dim,
+                                                        info),
                             rocsparse_status_invalid_pointer);
 
-    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrmv_ex<T>(handle,
-                                                  dir,
-                                                  trans,
-                                                  mb,
-                                                  nb,
-                                                  nnzb,
-                                                  alpha_device_host,
-                                                  descr,
-                                                  nullptr,
-                                                  bsr_row_ptr,
-                                                  nullptr,
-                                                  block_dim,
-                                                  info,
-                                                  x,
-                                                  beta_device_host,
-                                                  y),
+    EXPECT_ROCSPARSE_STATUS(rocsparse_bsrmv<T>(handle,
+                                               dir,
+                                               trans,
+                                               mb,
+                                               nb,
+                                               nnzb,
+                                               alpha_device_host,
+                                               descr,
+                                               nullptr,
+                                               bsr_row_ptr,
+                                               nullptr,
+                                               block_dim,
+                                               info,
+                                               x,
+                                               beta_device_host,
+                                               y),
                             rocsparse_status_invalid_pointer);
 }
 
 template <typename T>
 void testing_bsrmv(const Arguments& arg)
 {
-    rocsparse_int          M         = arg.M;
-    rocsparse_int          N         = arg.N;
-    rocsparse_direction    dir       = arg.direction;
-    rocsparse_operation    trans     = arg.transA;
-    rocsparse_index_base   base      = arg.baseA;
-    rocsparse_int          block_dim = arg.block_dim;
-    rocsparse_storage_mode storage   = arg.storage;
+    rocsparse_int          M                   = arg.M;
+    rocsparse_int          N                   = arg.N;
+    rocsparse_direction    dir                 = arg.direction;
+    rocsparse_operation    trans               = arg.transA;
+    rocsparse_index_base   base                = arg.baseA;
+    rocsparse_int          block_dim           = arg.block_dim;
+    rocsparse_storage_mode storage             = arg.storage;
+    const bool             call_stage_analysis = arg.call_stage_analysis;
 
-    host_scalar<T> h_alpha(arg.get_alpha<T>()), h_beta(arg.get_beta<T>());
+    host_scalar<T> h_alpha(arg.get_alpha<T>());
+    host_scalar<T> h_beta(arg.get_beta<T>());
+
+    device_scalar<T> d_alpha(h_alpha);
+    device_scalar<T> d_beta(h_beta);
 
     // Create rocsparse handle
     rocsparse_local_handle handle(arg);
@@ -166,12 +175,14 @@ void testing_bsrmv(const Arguments& arg)
     CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_storage_mode(descr, storage));
 
     // Create matrix info
-    rocsparse_local_mat_info info;
+
+    rocsparse_local_mat_info local_info;
+    rocsparse_mat_info       info = (call_stage_analysis) ? local_info : nullptr;
 
     // BSR dimensions
-    rocsparse_int mb = (block_dim > 0) ? (M + block_dim - 1) / block_dim : 0;
-    rocsparse_int nb = (block_dim > 0) ? (N + block_dim - 1) / block_dim : 0;
-    // Argument sanity check before allocating invalid memory
+    rocsparse_int mb = (M + block_dim - 1) / block_dim;
+    rocsparse_int nb = (N + block_dim - 1) / block_dim;
+
 #define PARAMS_ANALYSIS(A_)                                                                  \
     handle, A_.block_direction, trans, A_.mb, A_.nb, A_.nnzb, descr, A_.val, A_.ptr, A_.ind, \
         A_.row_block_dim, info
@@ -179,25 +190,6 @@ void testing_bsrmv(const Arguments& arg)
     handle, A_.block_direction, trans, A_.mb, A_.nb, A_.nnzb, alpha_, descr, A_.val, A_.ptr, \
         A_.ind, A_.row_block_dim, info, x_, beta_, y_
 
-    if(mb == 0 || nb == 0 || M == 0 || N == 0 || block_dim == 0)
-    {
-        device_gebsr_matrix<T> dA;
-        dA.block_direction = dir;
-        dA.mb              = mb;
-        dA.nb              = nb;
-        dA.nnzb            = 10;
-        dA.row_block_dim   = block_dim;
-        dA.col_block_dim   = block_dim;
-
-        device_dense_matrix<T> dx;
-        device_dense_matrix<T> dy;
-
-        CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
-        EXPECT_ROCSPARSE_STATUS(rocsparse_bsrmv_ex<T>(PARAMS(h_alpha, dA, dx, h_beta, dy)),
-                                rocsparse_status_success);
-        return;
-    }
-
     // Wavefront size
     int dev;
     CHECK_HIP_ERROR(hipGetDevice(&dev));
@@ -220,8 +212,8 @@ void testing_bsrmv(const Arguments& arg)
     }
     else
     {
-        int32_t block_row_dim = arg.row_block_dimA;
-        int32_t block_col_dim = arg.col_block_dimA;
+        rocsparse_int block_row_dim = arg.row_block_dimA;
+        rocsparse_int block_col_dim = arg.col_block_dimA;
         matrix_factory.init_gebsr(hA, M, N, block_row_dim, block_col_dim, base);
         dA(hA);
     }
@@ -238,13 +230,21 @@ void testing_bsrmv(const Arguments& arg)
     device_dense_matrix<T> dx(hx), dy(hy);
 
     // bsrmv_analysis (Optional)
-    CHECK_ROCSPARSE_ERROR(rocsparse_bsrmv_ex_analysis<T>(PARAMS_ANALYSIS(dA)));
+    if(call_stage_analysis)
+    {
+        CHECK_ROCSPARSE_ERROR(rocsparse_bsrmv_analysis<T>(PARAMS_ANALYSIS(dA)));
+    }
 
     if(arg.unit_check)
     {
         // Pointer mode host
         CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
-        CHECK_ROCSPARSE_ERROR(testing::rocsparse_bsrmv_ex<T>(PARAMS(h_alpha, dA, dx, h_beta, dy)));
+        CHECK_ROCSPARSE_ERROR(testing::rocsparse_bsrmv<T>(PARAMS(h_alpha, dA, dx, h_beta, dy)));
+
+        if(ROCSPARSE_REPRODUCIBILITY)
+        {
+            rocsparse_reproducibility::save("Y pointer mode host", dy);
+        }
 
         {
             host_dense_matrix<T> hy_copy(hy);
@@ -269,33 +269,22 @@ void testing_bsrmv(const Arguments& arg)
         }
 
         CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_device));
-        device_scalar<T> d_alpha(h_alpha), d_beta(h_beta);
-        CHECK_ROCSPARSE_ERROR(testing::rocsparse_bsrmv_ex<T>(PARAMS(d_alpha, dA, dx, d_beta, dy)));
+        CHECK_ROCSPARSE_ERROR(testing::rocsparse_bsrmv<T>(PARAMS(d_alpha, dA, dx, d_beta, dy)));
         hy.near_check(dy);
+
+        if(ROCSPARSE_REPRODUCIBILITY)
+        {
+            rocsparse_reproducibility::save("Y pointer mode device", dy);
+        }
     }
 
     if(arg.timing)
     {
-        int number_cold_calls = 2;
-        int number_hot_calls  = arg.iters;
 
         CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
 
-        // Warm up
-        for(int iter = 0; iter < number_cold_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_bsrmv_ex<T>(PARAMS(h_alpha, dA, dx, h_beta, dy)));
-        }
-
-        double gpu_time_used = get_time_us();
-
-        // Performance run
-        for(int iter = 0; iter < number_hot_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_bsrmv_ex<T>(PARAMS(h_alpha, dA, dx, h_beta, dy)));
-        }
-
-        gpu_time_used = (get_time_us() - gpu_time_used) / number_hot_calls;
+        const double gpu_time_used = rocsparse_clients::run_benchmark(
+            arg, rocsparse_bsrmv<T>, PARAMS(h_alpha, dA, dx, h_beta, dy));
 
         double gflop_count = spmv_gflop_count(
             M, dA.nnzb * dA.row_block_dim * dA.col_block_dim, *h_beta != static_cast<T>(0));
@@ -305,23 +294,23 @@ void testing_bsrmv(const Arguments& arg)
         double gpu_gflops = get_gpu_gflops(gpu_time_used, gflop_count);
         double gpu_gbyte  = get_gpu_gbyte(gpu_time_used, gbyte_count);
 
-        display_timing_info("M",
+        display_timing_info(display_key_t::M,
                             M,
-                            "N",
+                            display_key_t::N,
                             N,
-                            "BSR dim",
+                            display_key_t::bdim,
                             dA.row_block_dim,
-                            "dir",
+                            display_key_t::bdir,
                             rocsparse_direction2string(dA.block_direction),
-                            "alpha",
+                            display_key_t::alpha,
                             *h_alpha,
-                            "beta",
+                            display_key_t::beta,
                             *h_beta,
-                            s_timing_info_perf,
+                            display_key_t::gflops,
                             gpu_gflops,
-                            s_timing_info_bandwidth,
+                            display_key_t::bandwidth,
                             gpu_gbyte,
-                            s_timing_info_time,
+                            display_key_t::time_ms,
                             get_gpu_time_msec(gpu_time_used));
     }
 
