@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -166,42 +166,19 @@ void testing_bsrpad_value(const Arguments& arg)
 
     if(arg.timing)
     {
-        int number_cold_calls = 2;
-        int number_hot_calls  = arg.iters;
 
-        // Warm up
-        for(int iter = 0; iter < number_cold_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_bsrpad_value<T>(handle,
-                                                            M,
-                                                            Mb,
-                                                            dbsr.nnzb,
-                                                            block_dim,
-                                                            value,
-                                                            bsr_descr,
-                                                            dbsr.val,
-                                                            dbsr.ptr,
-                                                            dbsr.ind));
-        }
-
-        double gpu_time_used = get_time_us();
-
-        // Performance run
-        for(int iter = 0; iter < number_hot_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_bsrpad_value<T>(handle,
-                                                            M,
-                                                            Mb,
-                                                            dbsr.nnzb,
-                                                            block_dim,
-                                                            value,
-                                                            bsr_descr,
-                                                            dbsr.val,
-                                                            dbsr.ptr,
-                                                            dbsr.ind));
-        }
-
-        gpu_time_used = (get_time_us() - gpu_time_used) / number_hot_calls;
+        const double gpu_time_used = rocsparse_clients::run_benchmark(arg,
+                                                                      rocsparse_bsrpad_value<T>,
+                                                                      handle,
+                                                                      M,
+                                                                      Mb,
+                                                                      dbsr.nnzb,
+                                                                      block_dim,
+                                                                      value,
+                                                                      bsr_descr,
+                                                                      dbsr.val,
+                                                                      dbsr.ptr,
+                                                                      dbsr.ind);
 
         double gbyte_count = 0;
         double gpu_gbyte   = get_gpu_gbyte(gpu_time_used, gbyte_count);
