@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2021-2023 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -242,6 +242,7 @@ void testing_bsrsm_bad_arg(const Arguments& arg)
 template <typename T>
 void testing_bsrsm(const Arguments& arg)
 {
+    auto                      tol       = get_near_check_tol<T>(arg);
     rocsparse_int             m         = arg.M;
     rocsparse_int             nrhs      = arg.K;
     rocsparse_int             block_dim = arg.block_dim;
@@ -291,13 +292,13 @@ void testing_bsrsm(const Arguments& arg)
     // RHS matrix B
     host_dense_matrix<T> hB((trans_X == rocsparse_operation_none) ? m : nrhs,
                             (trans_X == rocsparse_operation_none) ? nrhs : m);
-    rocsparse_matrix_utils::init(hB);
+    rocsparse_matrix_utils::init(hB, arg.convert_to_int);
     device_dense_matrix<T> dB(hB);
 
     // Solution matrix X
     host_dense_matrix<T> hX_gold((trans_X == rocsparse_operation_none) ? m : nrhs,
                                  (trans_X == rocsparse_operation_none) ? nrhs : m);
-    rocsparse_matrix_utils::init(hX_gold);
+    rocsparse_matrix_utils::init(hX_gold, arg.convert_to_int);
     device_dense_matrix<T> dX(hX_gold);
 
     host_scalar<rocsparse_int> analysis_pivot_gold;
@@ -449,7 +450,7 @@ void testing_bsrsm(const Arguments& arg)
         // Check solution matrix if no pivot has been found
         if(*analysis_pivot_gold == -1 && *solve_pivot_gold == -1)
         {
-            hX_gold.near_check(dX);
+            hX_gold.near_check(dX, tol);
         }
 
         // DEVICE MODE
@@ -494,42 +495,62 @@ void testing_bsrsm(const Arguments& arg)
         // Check solution matrix if no pivot has been found
         if(*analysis_pivot_gold == -1 && *solve_pivot_gold == -1)
         {
-            hX_gold.near_check(dX);
+            hX_gold.near_check(dX, tol);
         }
     }
 
     if(arg.timing)
     {
-        int number_cold_calls = 2;
-        int number_hot_calls  = arg.iters;
+
+        device_dense_matrix<T> d_alpha(h_alpha);
 
         CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
 
-        // Warm up
-        for(int iter = 0; iter < number_cold_calls; ++iter)
-        {
-            CALL_ANALYSIS;
-            CALL_SOLVE(h_alpha);
-            CHECK_ROCSPARSE_ERROR(rocsparse_bsrsm_clear(handle, info));
-        }
-
-        double gpu_analysis_time_used = get_time_us();
-
-        CALL_ANALYSIS;
-
-        gpu_analysis_time_used = get_time_us() - gpu_analysis_time_used;
+        const double gpu_analysis_time_used
+            = rocsparse_clients::run_benchmark(arg,
+                                               rocsparse_bsrsm_analysis<T>,
+                                               handle,
+                                               dir,
+                                               trans_A,
+                                               trans_X,
+                                               mb,
+                                               nrhs,
+                                               dA.nnzb,
+                                               descr,
+                                               dA.val,
+                                               dA.ptr,
+                                               dA.ind,
+                                               block_dim,
+                                               info,
+                                               apol,
+                                               spol,
+                                               dbuffer);
 
         rocsparse_bsrsm_zero_pivot(handle, info, analysis_pivot_gold);
 
-        double gpu_solve_time_used = get_time_us();
-
-        // Performance run
-        for(int iter = 0; iter < number_hot_calls; ++iter)
-        {
-            CALL_SOLVE(h_alpha);
-        }
-
-        gpu_solve_time_used = (get_time_us() - gpu_solve_time_used) / number_hot_calls;
+        const double gpu_solve_time_used
+            = rocsparse_clients::run_benchmark(arg,
+                                               rocsparse_bsrsm_solve<T>,
+                                               handle,
+                                               dir,
+                                               trans_A,
+                                               trans_X,
+                                               mb,
+                                               nrhs,
+                                               dA.nnzb,
+                                               d_alpha,
+                                               descr,
+                                               dA.val,
+                                               dA.ptr,
+                                               dA.ind,
+                                               block_dim,
+                                               info,
+                                               dB,
+                                               dB.ld,
+                                               dX,
+                                               dX.ld,
+                                               spol,
+                                               dbuffer);
 
         rocsparse_bsrsm_zero_pivot(handle, info, solve_pivot_gold);
 
