@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -196,46 +196,21 @@ void testing_check_matrix_ell(const Arguments& arg)
 
     if(arg.timing)
     {
-        int number_cold_calls = 2;
-        int number_hot_calls  = arg.iters;
 
-        // Warm up
-        for(int iter = 0; iter < number_cold_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_check_matrix_ell<T>(handle,
-                                                                m,
-                                                                n,
-                                                                dA.width,
-                                                                dA.val,
-                                                                dA.ind,
-                                                                base,
-                                                                matrix_type,
-                                                                uplo,
-                                                                storage,
-                                                                &data_status,
-                                                                dbuffer));
-        }
-
-        double gpu_time_used = get_time_us();
-
-        // Performance run
-        for(int iter = 0; iter < number_hot_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_check_matrix_ell<T>(handle,
-                                                                m,
-                                                                n,
-                                                                dA.width,
-                                                                dA.val,
-                                                                dA.ind,
-                                                                base,
-                                                                matrix_type,
-                                                                uplo,
-                                                                storage,
-                                                                &data_status,
-                                                                dbuffer));
-        }
-
-        gpu_time_used = (get_time_us() - gpu_time_used) / number_hot_calls;
+        const double gpu_time_used = rocsparse_clients::run_benchmark(arg,
+                                                                      rocsparse_check_matrix_ell<T>,
+                                                                      handle,
+                                                                      m,
+                                                                      n,
+                                                                      dA.width,
+                                                                      dA.val,
+                                                                      dA.ind,
+                                                                      base,
+                                                                      matrix_type,
+                                                                      uplo,
+                                                                      storage,
+                                                                      &data_status,
+                                                                      dbuffer);
 
         double gbyte_count = check_matrix_ell_gbyte_count<T>(hA.nnz);
         double gpu_gbyte   = get_gpu_gbyte(gpu_time_used, gbyte_count);
