@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -305,47 +305,23 @@ void testing_csrcolor(const Arguments& arg)
 
     if(arg.timing)
     {
-        int number_cold_calls = 2;
-        int number_hot_calls  = arg.iters;
 
         CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
 
-        // Warm up
-        for(int iter = 0; iter < number_cold_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_csrcolor<T>(handle,
-                                                        dA.m,
-                                                        dA.nnz,
-                                                        csr_descr,
-                                                        dA.val,
-                                                        dA.ptr,
-                                                        dA.ind,
-                                                        &fraction_to_color,
-                                                        &ncolor,
-                                                        dcoloring,
-                                                        dreordering,
-                                                        mat_info));
-        }
-
-        double gpu_time_used = get_time_us();
-        // Performance run
-        for(int iter = 0; iter < number_hot_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_csrcolor<T>(handle,
-                                                        dA.m,
-                                                        dA.nnz,
-                                                        csr_descr,
-                                                        dA.val,
-                                                        dA.ptr,
-                                                        dA.ind,
-                                                        &fraction_to_color,
-                                                        &ncolor,
-                                                        dcoloring,
-                                                        dreordering,
-                                                        mat_info));
-        }
-
-        gpu_time_used = (get_time_us() - gpu_time_used) / number_hot_calls;
+        const double gpu_time_used = rocsparse_clients::run_benchmark(arg,
+                                                                      rocsparse_csrcolor<T>,
+                                                                      handle,
+                                                                      dA.m,
+                                                                      dA.nnz,
+                                                                      csr_descr,
+                                                                      dA.val,
+                                                                      dA.ptr,
+                                                                      dA.ind,
+                                                                      &fraction_to_color,
+                                                                      &ncolor,
+                                                                      dcoloring,
+                                                                      dreordering,
+                                                                      mat_info);
 
         display_timing_info(display_key_t::M,
                             dA.m,
