@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
-* Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
+* Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -170,28 +170,11 @@ void testing_gemmi(const Arguments& arg)
 
     if(arg.timing)
     {
-        int number_cold_calls = 2;
-        int number_hot_calls  = arg.iters;
 
         CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
 
-        // Warm up
-        for(int iter = 0; iter < number_cold_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(
-                testing::rocsparse_gemmi<T>(PARAMS(transA, transB, h_alpha, dA, dB, h_beta, dC)));
-        }
-
-        double gpu_time_used = get_time_us();
-
-        // Performance run
-        for(int iter = 0; iter < number_hot_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(
-                testing::rocsparse_gemmi<T>(PARAMS(transA, transB, h_alpha, dA, dB, h_beta, dC)));
-        }
-
-        gpu_time_used = (get_time_us() - gpu_time_used) / number_hot_calls;
+        const double gpu_time_used = rocsparse_clients::run_benchmark(
+            arg, rocsparse_gemmi<T>, PARAMS(transA, transB, h_alpha, dA, dB, h_beta, dC));
 
         double gpu_gflops = get_gpu_gflops(gpu_time_used,
                                            csrmm_gflop_count<rocsparse_int, rocsparse_int>,
@@ -200,7 +183,7 @@ void testing_gemmi(const Arguments& arg)
                                            nnz_C,
                                            *h_beta != static_cast<T>(0));
         double gpu_gbyte  = get_gpu_gbyte(gpu_time_used,
-                                         csrmm_gbyte_count<T, rocsparse_int, rocsparse_int>,
+                                         csrmm_gbyte_count<T, T, T, rocsparse_int, rocsparse_int>,
                                          hB.m,
                                          hB.nnz,
                                          nnz_A,
