@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2023 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -276,52 +276,25 @@ void testing_prune_csr2csr(const Arguments& arg)
 
     if(arg.timing)
     {
-        int number_cold_calls = 2;
-        int number_hot_calls  = arg.iters;
 
         CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
 
-        // Warm up
-        for(int iter = 0; iter < number_cold_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_prune_csr2csr<T>(handle,
-                                                             M,
-                                                             N,
-                                                             nnz_A,
-                                                             csr_descr_A,
-                                                             d_csr_val_A,
-                                                             d_csr_row_ptr_A,
-                                                             d_csr_col_ind_A,
-                                                             h_threshold,
-                                                             csr_descr_C,
-                                                             d_csr_val_C,
-                                                             d_csr_row_ptr_C,
-                                                             d_csr_col_ind_C,
-                                                             d_temp_buffer));
-        }
-
-        double gpu_time_used = get_time_us();
-
-        // Performance run
-        for(int iter = 0; iter < number_hot_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_prune_csr2csr<T>(handle,
-                                                             M,
-                                                             N,
-                                                             nnz_A,
-                                                             csr_descr_A,
-                                                             d_csr_val_A,
-                                                             d_csr_row_ptr_A,
-                                                             d_csr_col_ind_A,
-                                                             h_threshold,
-                                                             csr_descr_C,
-                                                             d_csr_val_C,
-                                                             d_csr_row_ptr_C,
-                                                             d_csr_col_ind_C,
-                                                             d_temp_buffer));
-        }
-
-        gpu_time_used = (get_time_us() - gpu_time_used) / number_hot_calls;
+        const double gpu_time_used = rocsparse_clients::run_benchmark(arg,
+                                                                      rocsparse_prune_csr2csr<T>,
+                                                                      handle,
+                                                                      M,
+                                                                      N,
+                                                                      nnz_A,
+                                                                      csr_descr_A,
+                                                                      d_csr_val_A,
+                                                                      d_csr_row_ptr_A,
+                                                                      d_csr_col_ind_A,
+                                                                      h_threshold,
+                                                                      csr_descr_C,
+                                                                      d_csr_val_C,
+                                                                      d_csr_row_ptr_C,
+                                                                      d_csr_col_ind_C,
+                                                                      d_temp_buffer);
 
         double gbyte_count = prune_csr2csr_gbyte_count<T>(M, nnz_A, h_nnz_total_dev_host_ptr[0]);
         double gpu_gbyte   = get_gpu_gbyte(gpu_time_used, gbyte_count);
