@@ -1,5 +1,5 @@
 /* ************************************************************************
-* Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights Reserved.
+* Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -230,7 +230,14 @@ void testing_spmm_batched_coo(const Arguments& arg)
         {
             hcoo_row_ind[nnz_A * i + j] = hcoo_row_ind_temp[j];
             hcoo_col_ind[nnz_A * i + j] = hcoo_col_ind_temp[j];
-            hcoo_val[nnz_A * i + j]     = hcoo_val_temp[j];
+            if(arg.convert_to_int)
+            {
+                hcoo_val[nnz_A * i + j] = convertToInt<A>(hcoo_val_temp[j]);
+            }
+            else
+            {
+                hcoo_val[nnz_A * i + j] = hcoo_val_temp[j];
+            }
         }
     }
 
@@ -239,8 +246,16 @@ void testing_spmm_batched_coo(const Arguments& arg)
     host_vector<C> hC_1(batch_count_C * nnz_C);
 
     // Initialize data on CPU
-    rocsparse_init<B>(hB, batch_count_B * nnz_B, 1, 1);
-    rocsparse_init<C>(hC_1, batch_count_C * nnz_C, 1, 1);
+    rocsparse_init<B>(hB, batch_count_B * nnz_B, 1, 1, arg.convert_to_int);
+    rocsparse_init<C>(hC_1, batch_count_C * nnz_C, 1, 1, arg.convert_to_int);
+
+    if(arg.convert_to_int)
+    {
+        for(I i = 0; i < batch_count_B * nnz_B; i++)
+        {
+            hB[i] = convertToInt<B>(hB[i]);
+        }
+    }
 
     host_vector<C> hC_2(hC_1);
     host_vector<C> hC_gold(hC_1);
@@ -378,64 +393,38 @@ void testing_spmm_batched_coo(const Arguments& arg)
 
     if(arg.timing)
     {
-        int number_cold_calls = 2;
-        int number_hot_calls  = arg.iters;
 
         CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
 
-        // Warm up
-        for(int iter = 0; iter < number_cold_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_spmm(handle,
-                                                 trans_A,
-                                                 trans_B,
-                                                 &halpha,
-                                                 mat_A,
-                                                 mat_B,
-                                                 &hbeta,
-                                                 mat_C1,
-                                                 ttype,
-                                                 alg,
-                                                 rocsparse_spmm_stage_compute,
-                                                 &buffer_size,
-                                                 dbuffer));
-        }
-
-        double gpu_time_used = get_time_us();
-
-        // Performance run
-        for(int iter = 0; iter < number_hot_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_spmm(handle,
-                                                 trans_A,
-                                                 trans_B,
-                                                 &halpha,
-                                                 mat_A,
-                                                 mat_B,
-                                                 &hbeta,
-                                                 mat_C1,
-                                                 ttype,
-                                                 alg,
-                                                 rocsparse_spmm_stage_compute,
-                                                 &buffer_size,
-                                                 dbuffer));
-        }
-
-        gpu_time_used = (get_time_us() - gpu_time_used) / number_hot_calls;
+        const double gpu_time_used = rocsparse_clients::run_benchmark(arg,
+                                                                      rocsparse_spmm,
+                                                                      handle,
+                                                                      trans_A,
+                                                                      trans_B,
+                                                                      &halpha,
+                                                                      mat_A,
+                                                                      mat_B,
+                                                                      &hbeta,
+                                                                      mat_C1,
+                                                                      ttype,
+                                                                      alg,
+                                                                      rocsparse_spmm_stage_compute,
+                                                                      &buffer_size,
+                                                                      dbuffer);
 
         double gflop_count
             = batch_count_C
               * spmm_gflop_count(N, nnz_A, (int64_t)C_m * (int64_t)C_n, hbeta != static_cast<T>(0));
         double gpu_gflops = get_gpu_gflops(gpu_time_used, gflop_count);
 
-        double gbyte_count = coomm_batched_gbyte_count<T>(A_m,
-                                                          nnz_A,
-                                                          (int64_t)B_m * (int64_t)B_n,
-                                                          (int64_t)C_m * (int64_t)C_n,
-                                                          batch_count_A,
-                                                          batch_count_B,
-                                                          batch_count_C,
-                                                          hbeta != static_cast<T>(0));
+        double gbyte_count = coomm_batched_gbyte_count<A, B, C>(A_m,
+                                                                nnz_A,
+                                                                (int64_t)B_m * (int64_t)B_n,
+                                                                (int64_t)C_m * (int64_t)C_n,
+                                                                batch_count_A,
+                                                                batch_count_B,
+                                                                batch_count_C,
+                                                                hbeta != static_cast<T>(0));
         double gpu_gbyte   = get_gpu_gbyte(gpu_time_used, gbyte_count);
 
         display_timing_info(display_key_t::M,
@@ -491,5 +480,9 @@ INSTANTIATE_MIXED(int32_t, int8_t, int8_t, int32_t, int32_t);
 INSTANTIATE_MIXED(int64_t, int8_t, int8_t, int32_t, int32_t);
 INSTANTIATE_MIXED(int32_t, int8_t, int8_t, float, float);
 INSTANTIATE_MIXED(int64_t, int8_t, int8_t, float, float);
+INSTANTIATE_MIXED(int32_t, _Float16, _Float16, float, float);
+INSTANTIATE_MIXED(int64_t, _Float16, _Float16, float, float);
+INSTANTIATE_MIXED(int32_t, rocsparse_bfloat16, rocsparse_bfloat16, float, float);
+INSTANTIATE_MIXED(int64_t, rocsparse_bfloat16, rocsparse_bfloat16, float, float);
 
 void testing_spmm_batched_coo_extra(const Arguments& arg) {}
