@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
-* Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights Reserved.
+* Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -138,6 +138,10 @@ static constexpr std::size_t countof2(T (&)[N])
   TRANSFORM_ROCSPARSE_TEST_ENUM(spgemm_bsr)				\
   TRANSFORM_ROCSPARSE_TEST_ENUM(spgemm_csr)				\
   TRANSFORM_ROCSPARSE_TEST_ENUM(spgemm_reuse_csr)				\
+  TRANSFORM_ROCSPARSE_TEST_ENUM(spgeam_csr)				\
+  TRANSFORM_ROCSPARSE_TEST_ENUM(spgeam_reuse_csr)				\
+  TRANSFORM_ROCSPARSE_TEST_ENUM(spgeam_csr_2)				\
+  TRANSFORM_ROCSPARSE_TEST_ENUM(spgeam_reuse_csr_2)				\
   TRANSFORM_ROCSPARSE_TEST_ENUM(spmat_descr)				\
   TRANSFORM_ROCSPARSE_TEST_ENUM(spmm_bell)				\
   TRANSFORM_ROCSPARSE_TEST_ENUM(spmm_bsr)				\
@@ -154,6 +158,12 @@ static constexpr std::size_t countof2(T (&)[N])
   TRANSFORM_ROCSPARSE_TEST_ENUM(spmv_csr)				\
   TRANSFORM_ROCSPARSE_TEST_ENUM(spmv_csc)				\
   TRANSFORM_ROCSPARSE_TEST_ENUM(spmv_ell)				\
+  TRANSFORM_ROCSPARSE_TEST_ENUM(v2_spmv_bsr)				\
+  TRANSFORM_ROCSPARSE_TEST_ENUM(v2_spmv_coo_aos)				\
+  TRANSFORM_ROCSPARSE_TEST_ENUM(v2_spmv_coo)				\
+  TRANSFORM_ROCSPARSE_TEST_ENUM(v2_spmv_csr)				\
+  TRANSFORM_ROCSPARSE_TEST_ENUM(v2_spmv_csc)				\
+  TRANSFORM_ROCSPARSE_TEST_ENUM(v2_spmv_ell)				\
   TRANSFORM_ROCSPARSE_TEST_ENUM(spsm_coo)				\
   TRANSFORM_ROCSPARSE_TEST_ENUM(spsm_csr)				\
   TRANSFORM_ROCSPARSE_TEST_ENUM(spsv_coo)				\
