@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2019-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2019-2023 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -26,4 +26,4 @@
 
 #include "testing_csricsv.hpp"
 
-TEST_ROUTINE(csricsv, extra, arg.apol, arg.spol, arg.baseA, arg.matrix, arg.filename);
+TEST_ROUTINE(csricsv, extra, arg.apol, arg.spol, arg.baseA, arg.matrix);
