@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2021-2022 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2021-2023 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -26,5 +26,14 @@
 
 #include "testing_gemvi.hpp"
 
-TEST_ROUTINE(
-    gemvi, level2, arg.M, arg.N, arg.alpha, arg.alphai, arg.beta, arg.betai, arg.transA, arg.baseA);
+TEST_ROUTINE(gemvi,
+             level2,
+             arg.M,
+             arg.N,
+             arg.alpha,
+             arg.alphai,
+             arg.beta,
+             arg.betai,
+             arg.transA,
+             arg.baseA,
+             arg.graph_test);
