@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2024-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -33,7 +33,6 @@
 
 namespace rocsparseio
 {
-
     ///
     ///
     ///
@@ -169,6 +168,12 @@ inline std::ostream& operator<<(std::ostream& os, const rocsparseio::file_format
     return os;
 }
 
+inline std::ostream& operator<<(std::ostream& os, const _Float16& that_)
+{
+    os << (float)that_;
+    return os;
+}
+
 struct mtx_header
 {
     char banner[16];
@@ -680,6 +685,10 @@ rocsparseio_status write_ascii_sparse_coo_dispatch_t(rocsparseio_type t, P... pa
 {
     switch(t)
     {
+    case rocsparseio_type_float16:
+    {
+        return write_ascii_sparse_coo_template<I, J, _Float16>(params...);
+    }
     case rocsparseio_type_float32:
     {
         return write_ascii_sparse_coo_template<I, J, float>(params...);
@@ -702,6 +711,7 @@ rocsparseio_status write_ascii_sparse_coo_dispatch_t(rocsparseio_type t, P... pa
     }
     case rocsparseio_type_int32:
     case rocsparseio_type_int64:
+    case rocsparseio_type_bfloat16:
     {
         return rocsparseio_status_invalid_value;
     }
@@ -723,6 +733,8 @@ rocsparseio_status
         return write_ascii_sparse_coo_dispatch_t<I, int64_t, P...>(t, params...);
     }
     case rocsparseio_type_int8:
+    case rocsparseio_type_float16:
+    case rocsparseio_type_bfloat16:
     case rocsparseio_type_float32:
     case rocsparseio_type_float64:
     case rocsparseio_type_complex32:
@@ -748,6 +760,8 @@ rocsparseio_status
         return write_ascii_sparse_coo_dispatch_jt<int64_t, P...>(j, t, params...);
     }
     case rocsparseio_type_int8:
+    case rocsparseio_type_float16:
+    case rocsparseio_type_bfloat16:
     case rocsparseio_type_float32:
     case rocsparseio_type_float64:
     case rocsparseio_type_complex32:
@@ -775,6 +789,10 @@ rocsparseio_status write_ascii_dense_vector(rocsparseio_type t, P... params)
     {
         return write_ascii_dense_vector_template<int8_t>(params...);
     }
+    case rocsparseio_type_float16:
+    {
+        return write_ascii_dense_vector_template<_Float16>(params...);
+    }
     case rocsparseio_type_float32:
     {
         return write_ascii_dense_vector_template<float>(params...);
@@ -791,6 +809,10 @@ rocsparseio_status write_ascii_dense_vector(rocsparseio_type t, P... params)
     {
         return write_ascii_dense_vector_template<std::complex<double>>(params...);
     }
+    case rocsparseio_type_bfloat16:
+    {
+        return rocsparseio_status_invalid_value;
+    }
     }
 }
 
@@ -1058,6 +1080,8 @@ rocsparseio_status rocsparseio2csr(const char* ifilename, const char* ofilename)
             break;
         }
         case rocsparseio_type_int8:
+        case rocsparseio_type_float16:
+        case rocsparseio_type_bfloat16:
         case rocsparseio_type_float32:
         case rocsparseio_type_float64:
         case rocsparseio_type_complex32:
@@ -1087,6 +1111,8 @@ rocsparseio_status rocsparseio2csr(const char* ifilename, const char* ofilename)
             break;
         }
         case rocsparseio_type_int8:
+        case rocsparseio_type_float16:
+        case rocsparseio_type_bfloat16:
         case rocsparseio_type_float32:
         case rocsparseio_type_float64:
         case rocsparseio_type_complex32:
@@ -1115,11 +1141,22 @@ rocsparseio_status rocsparseio2csr(const char* ifilename, const char* ofilename)
             }
             break;
         }
+        case rocsparseio_type_float16:
+        {
+            idata       = (double*)malloc(sizeof(double) * (nnz));
+            _Float16* p = (_Float16*)data;
+            for(uint64_t i = 0; i < nnz; ++i)
+            {
+                idata[i] = p[i];
+            }
+            break;
+        }
         case rocsparseio_type_int8:
         case rocsparseio_type_int32:
         case rocsparseio_type_int64:
         case rocsparseio_type_complex32:
         case rocsparseio_type_complex64:
+        case rocsparseio_type_bfloat16:
         {
             break;
         }
