@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -26,7 +26,7 @@
 
 #include <limits>
 
-#include "common.h"
+#include "rocsparse_common.hpp"
 
 namespace rocsparse
 {
@@ -47,7 +47,6 @@ namespace rocsparse
     template <rocsparse_int BLOCK_SIZE,
               rocsparse_int SEGMENTS_PER_BLOCK,
               rocsparse_int SEGMENT_SIZE,
-              rocsparse_int WF_SIZE,
               typename T>
     ROCSPARSE_DEVICE_ILF void nnz_compress_device(rocsparse_int        m,
                                                   rocsparse_index_base idx_base_A,
@@ -83,7 +82,7 @@ namespace rocsparse
             count = rocsparse::wfreduce_sum<SEGMENT_SIZE>(count);
 
             // broadcast count from last thread in segment to all threads in segment
-            count = __shfl(count, SEGMENT_SIZE - 1, SEGMENT_SIZE);
+            count = rocsparse::shfl(count, SEGMENT_SIZE - 1, SEGMENT_SIZE);
 
             nnz_per_row[row_index] = count;
         }
