@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2018-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,7 +22,7 @@
  *
  * ************************************************************************ */
 #include "internal/conversion/rocsparse_inverse_permutation.h"
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
 #include "rocsparse_gcreate_identity_permutation.hpp"
 #include "rocsparse_identity.hpp"
@@ -32,6 +32,8 @@
 template <typename I>
 rocsparse_status rocsparse::create_identity_permutation_core(rocsparse_handle handle, I n, I* p)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     // Stream
     hipStream_t stream = handle->stream;
 
@@ -54,6 +56,8 @@ rocsparse_status rocsparse::create_identity_permutation_core(rocsparse_handle ha
 template <typename I>
 rocsparse_status rocsparse::create_identity_permutation_template(rocsparse_handle handle, I n, I* p)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     // Quick return if possible
     if(n == 0)
     {
@@ -66,6 +70,8 @@ rocsparse_status rocsparse::create_identity_permutation_template(rocsparse_handl
 template <typename I>
 rocsparse_status rocsparse::create_identity_permutation_impl(rocsparse_handle handle, I n, I* p)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     // Logging
     rocsparse::log_trace(handle, "rocsparse_create_identity_permutation", n, (const void*&)p);
 
@@ -99,13 +105,17 @@ extern "C" rocsparse_status rocsparse_create_identity_permutation(rocsparse_hand
                                                                   rocsparse_int*   p)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::create_identity_permutation_impl(handle, n, p));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status rocsparse_set_identity_permutation(rocsparse_handle    handle,
                                                                int64_t             n,
@@ -113,10 +123,14 @@ extern "C" rocsparse_status rocsparse_set_identity_permutation(rocsparse_handle
                                                                rocsparse_indextype indextype)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::gcreate_identity_permutation(handle, n, indextype, p));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
