@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,7 +22,7 @@
  * ************************************************************************ */
 
 #include "internal/generic/rocsparse_rot.h"
-#include "control.h"
+#include "rocsparse_control.hpp"
 #include "rocsparse_roti.hpp"
 
 namespace rocsparse
@@ -34,6 +34,8 @@ namespace rocsparse
                                   rocsparse_spvec_descr x,
                                   rocsparse_dnvec_descr y)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         RETURN_IF_ROCSPARSE_ERROR((rocsparse::roti_template<I, T>)(handle,
                                                                    (I)x->nnz,
                                                                    (T*)x->val_data,
@@ -59,6 +61,8 @@ extern "C" rocsparse_status rocsparse_rot(rocsparse_handle      handle,
                                           rocsparse_dnvec_descr y)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     // Check for invalid handle
     ROCSPARSE_CHECKARG_HANDLE(0, handle);
 
@@ -136,9 +140,11 @@ try
         return rocsparse_status_success;
     }
 
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
