@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,7 +23,7 @@
  * ************************************************************************ */
 
 #include "bsrmm_device_general.h"
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
 namespace rocsparse
 {
@@ -34,7 +34,7 @@ namespace rocsparse
               typename A,
               typename B,
               typename C,
-              typename U>
+              typename T>
     ROCSPARSE_KERNEL(BSR_BLOCK_DIM* BLK_SIZE_Y)
     void bsrmm_general_blockdim_kernel(bool                nn,
                                        rocsparse_direction direction,
@@ -42,7 +42,7 @@ namespace rocsparse
                                        J                   n,
                                        int64_t             offsets_batch_stride_A,
                                        int64_t             columns_values_batch_stride_A,
-                                       U                   alpha_device_host,
+                                       ROCSPARSE_DEVICE_HOST_SCALAR_PARAMS(T, alpha),
                                        const I* __restrict__ bsr_row_ptr,
                                        const J* __restrict__ bsr_col_ind,
                                        const A* __restrict__ bsr_val,
@@ -50,15 +50,16 @@ namespace rocsparse
                                        const B* __restrict__ dense_B,
                                        int64_t ldb,
                                        int64_t batch_stride_B,
-                                       U       beta_device_host,
+                                       ROCSPARSE_DEVICE_HOST_SCALAR_PARAMS(T, beta),
                                        C* __restrict__ dense_C,
                                        int64_t              ldc,
                                        int64_t              batch_stride_C,
                                        rocsparse_order      order_C,
-                                       rocsparse_index_base idx_base)
+                                       rocsparse_index_base idx_base,
+                                       bool                 is_host_mode)
     {
-        const auto alpha = rocsparse::load_scalar_device_host(alpha_device_host);
-        const auto beta  = rocsparse::load_scalar_device_host(beta_device_host);
+        ROCSPARSE_DEVICE_HOST_SCALAR_GET(alpha);
+        ROCSPARSE_DEVICE_HOST_SCALAR_GET(beta);
 
         if(alpha == 0 && beta == 1)
         {
@@ -88,7 +89,7 @@ namespace rocsparse
             idx_base);
     }
 
-    template <typename T, typename I, typename J, typename A, typename B, typename C, typename U>
+    template <typename T, typename I, typename J, typename A, typename B, typename C>
     rocsparse_status bsrmm_template_general(bool                      nn,
                                             rocsparse_handle          handle,
                                             rocsparse_direction       dir,
@@ -101,7 +102,7 @@ namespace rocsparse
                                             J                         batch_count_A,
                                             int64_t                   offsets_batch_stride_A,
                                             int64_t                   columns_values_batch_stride_A,
-                                            U                         alpha,
+                                            const T*                  alpha,
                                             const rocsparse_mat_descr descr,
                                             const A*                  bsr_val,
                                             const I*                  bsr_row_ptr,
@@ -112,13 +113,15 @@ namespace rocsparse
                                             J                         batch_count_B,
                                             int64_t                   batch_stride_B,
                                             rocsparse_order           order_B,
-                                            U                         beta,
+                                            const T*                  beta,
                                             C*                        dense_C,
                                             int64_t                   ldc,
                                             J                         batch_count_C,
                                             int64_t                   batch_stride_C,
                                             rocsparse_order           order_C)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         hipStream_t stream = handle->stream;
         rocsparse_host_assert(block_dim > 32, "This function is designed for block_dim > 32.");
 
@@ -135,7 +138,7 @@ namespace rocsparse
                                            n,
                                            offsets_batch_stride_A,
                                            columns_values_batch_stride_A,
-                                           alpha,
+                                           ROCSPARSE_DEVICE_HOST_SCALAR_ARGS(handle, alpha),
                                            bsr_row_ptr,
                                            bsr_col_ind,
                                            bsr_val,
@@ -143,160 +146,106 @@ namespace rocsparse
                                            dense_B,
                                            ldb,
                                            batch_stride_B,
-                                           beta,
+                                           ROCSPARSE_DEVICE_HOST_SCALAR_ARGS(handle, beta),
                                            dense_C,
                                            ldc,
                                            batch_stride_C,
                                            order_C,
-                                           descr->base);
+                                           descr->base,
+                                           handle->pointer_mode == rocsparse_pointer_mode_host);
 
         return rocsparse_status_success;
     }
 }
 
-#define INSTANTIATE(TTYPE, ITYPE, JTYPE, ATYPE, BTYPE, CTYPE, UTYPE)    \
-    template rocsparse_status rocsparse::bsrmm_template_general<TTYPE>( \
-        bool                      nn,                                   \
-        rocsparse_handle          handle,                               \
-        rocsparse_direction       dir,                                  \
-        rocsparse_operation       trans_A,                              \
-        rocsparse_operation       trans_B,                              \
-        JTYPE                     mb,                                   \
-        JTYPE                     n,                                    \
-        JTYPE                     kb,                                   \
-        ITYPE                     nnzb,                                 \
-        JTYPE                     batch_count_A,                        \
-        int64_t                   offsets_batch_stride_A,               \
-        int64_t                   columns_values_batch_stride_A,        \
-        UTYPE                     alpha,                                \
-        const rocsparse_mat_descr descr,                                \
-        const ATYPE*              bsr_val,                              \
-        const ITYPE*              bsr_row_ptr,                          \
-        const JTYPE*              bsr_col_ind,                          \
-        JTYPE                     block_dim,                            \
-        const BTYPE*              dense_B,                              \
-        int64_t                   ldb,                                  \
-        JTYPE                     batch_count_B,                        \
-        int64_t                   batch_stride_B,                       \
-        rocsparse_order           order_B,                              \
-        UTYPE                     beta,                                 \
-        CTYPE*                    dense_C,                              \
-        int64_t                   ldc,                                  \
-        JTYPE                     batch_count_C,                        \
-        int64_t                   batch_stride_C,                       \
+#define INSTANTIATE(TTYPE, ITYPE, JTYPE, ATYPE, BTYPE, CTYPE)    \
+    template rocsparse_status rocsparse::bsrmm_template_general( \
+        bool                      nn,                            \
+        rocsparse_handle          handle,                        \
+        rocsparse_direction       dir,                           \
+        rocsparse_operation       trans_A,                       \
+        rocsparse_operation       trans_B,                       \
+        JTYPE                     mb,                            \
+        JTYPE                     n,                             \
+        JTYPE                     kb,                            \
+        ITYPE                     nnzb,                          \
+        JTYPE                     batch_count_A,                 \
+        int64_t                   offsets_batch_stride_A,        \
+        int64_t                   columns_values_batch_stride_A, \
+        const TTYPE*              alpha,                         \
+        const rocsparse_mat_descr descr,                         \
+        const ATYPE*              bsr_val,                       \
+        const ITYPE*              bsr_row_ptr,                   \
+        const JTYPE*              bsr_col_ind,                   \
+        JTYPE                     block_dim,                     \
+        const BTYPE*              dense_B,                       \
+        int64_t                   ldb,                           \
+        JTYPE                     batch_count_B,                 \
+        int64_t                   batch_stride_B,                \
+        rocsparse_order           order_B,                       \
+        const TTYPE*              beta,                          \
+        CTYPE*                    dense_C,                       \
+        int64_t                   ldc,                           \
+        JTYPE                     batch_count_C,                 \
+        int64_t                   batch_stride_C,                \
         rocsparse_order           order_C)
 
 // Uniform precisions
-INSTANTIATE(float, int32_t, int32_t, float, float, float, float);
-INSTANTIATE(float, int64_t, int32_t, float, float, float, float);
-INSTANTIATE(float, int64_t, int64_t, float, float, float, float);
-INSTANTIATE(double, int32_t, int32_t, double, double, double, double);
-INSTANTIATE(double, int64_t, int32_t, double, double, double, double);
-INSTANTIATE(double, int64_t, int64_t, double, double, double, double);
+INSTANTIATE(float, int32_t, int32_t, float, float, float);
+INSTANTIATE(float, int64_t, int32_t, float, float, float);
+INSTANTIATE(float, int64_t, int64_t, float, float, float);
+INSTANTIATE(double, int32_t, int32_t, double, double, double);
+INSTANTIATE(double, int64_t, int32_t, double, double, double);
+INSTANTIATE(double, int64_t, int64_t, double, double, double);
 INSTANTIATE(rocsparse_float_complex,
             int32_t,
             int32_t,
             rocsparse_float_complex,
             rocsparse_float_complex,
-            rocsparse_float_complex,
             rocsparse_float_complex);
 INSTANTIATE(rocsparse_float_complex,
             int64_t,
             int32_t,
             rocsparse_float_complex,
             rocsparse_float_complex,
-            rocsparse_float_complex,
             rocsparse_float_complex);
 INSTANTIATE(rocsparse_float_complex,
             int64_t,
             int64_t,
             rocsparse_float_complex,
             rocsparse_float_complex,
-            rocsparse_float_complex,
             rocsparse_float_complex);
 INSTANTIATE(rocsparse_double_complex,
             int32_t,
             int32_t,
             rocsparse_double_complex,
             rocsparse_double_complex,
-            rocsparse_double_complex,
             rocsparse_double_complex);
 INSTANTIATE(rocsparse_double_complex,
             int64_t,
             int32_t,
             rocsparse_double_complex,
             rocsparse_double_complex,
-            rocsparse_double_complex,
             rocsparse_double_complex);
 INSTANTIATE(rocsparse_double_complex,
             int64_t,
             int64_t,
             rocsparse_double_complex,
             rocsparse_double_complex,
-            rocsparse_double_complex,
             rocsparse_double_complex);
 
-INSTANTIATE(float, int32_t, int32_t, float, float, float, const float*);
-INSTANTIATE(float, int64_t, int32_t, float, float, float, const float*);
-INSTANTIATE(float, int64_t, int64_t, float, float, float, const float*);
-INSTANTIATE(double, int32_t, int32_t, double, double, double, const double*);
-INSTANTIATE(double, int64_t, int32_t, double, double, double, const double*);
-INSTANTIATE(double, int64_t, int64_t, double, double, double, const double*);
-INSTANTIATE(rocsparse_float_complex,
-            int32_t,
-            int32_t,
-            rocsparse_float_complex,
-            rocsparse_float_complex,
-            rocsparse_float_complex,
-            const rocsparse_float_complex*);
-INSTANTIATE(rocsparse_float_complex,
-            int64_t,
-            int32_t,
-            rocsparse_float_complex,
-            rocsparse_float_complex,
-            rocsparse_float_complex,
-            const rocsparse_float_complex*);
-INSTANTIATE(rocsparse_float_complex,
-            int64_t,
-            int64_t,
-            rocsparse_float_complex,
-            rocsparse_float_complex,
-            rocsparse_float_complex,
-            const rocsparse_float_complex*);
-INSTANTIATE(rocsparse_double_complex,
-            int32_t,
-            int32_t,
-            rocsparse_double_complex,
-            rocsparse_double_complex,
-            rocsparse_double_complex,
-            const rocsparse_double_complex*);
-INSTANTIATE(rocsparse_double_complex,
-            int64_t,
-            int32_t,
-            rocsparse_double_complex,
-            rocsparse_double_complex,
-            rocsparse_double_complex,
-            const rocsparse_double_complex*);
-INSTANTIATE(rocsparse_double_complex,
-            int64_t,
-            int64_t,
-            rocsparse_double_complex,
-            rocsparse_double_complex,
-            rocsparse_double_complex,
-            const rocsparse_double_complex*);
-
 // Mixed Precisions
-INSTANTIATE(int32_t, int32_t, int32_t, int8_t, int8_t, int32_t, int32_t);
-INSTANTIATE(int32_t, int64_t, int32_t, int8_t, int8_t, int32_t, int32_t);
-INSTANTIATE(int32_t, int64_t, int64_t, int8_t, int8_t, int32_t, int32_t);
-INSTANTIATE(float, int32_t, int32_t, int8_t, int8_t, float, float);
-INSTANTIATE(float, int64_t, int32_t, int8_t, int8_t, float, float);
-INSTANTIATE(float, int64_t, int64_t, int8_t, int8_t, float, float);
+INSTANTIATE(float, int32_t, int32_t, _Float16, _Float16, float);
+INSTANTIATE(float, int64_t, int32_t, _Float16, _Float16, float);
+INSTANTIATE(float, int64_t, int64_t, _Float16, _Float16, float);
+INSTANTIATE(float, int32_t, int32_t, rocsparse_bfloat16, rocsparse_bfloat16, float);
+INSTANTIATE(float, int64_t, int32_t, rocsparse_bfloat16, rocsparse_bfloat16, float);
+INSTANTIATE(float, int64_t, int64_t, rocsparse_bfloat16, rocsparse_bfloat16, float);
+INSTANTIATE(int32_t, int32_t, int32_t, int8_t, int8_t, int32_t);
+INSTANTIATE(int32_t, int64_t, int32_t, int8_t, int8_t, int32_t);
+INSTANTIATE(int32_t, int64_t, int64_t, int8_t, int8_t, int32_t);
+INSTANTIATE(float, int32_t, int32_t, int8_t, int8_t, float);
+INSTANTIATE(float, int64_t, int32_t, int8_t, int8_t, float);
+INSTANTIATE(float, int64_t, int64_t, int8_t, int8_t, float);
 
-INSTANTIATE(int32_t, int32_t, int32_t, int8_t, int8_t, int32_t, const int32_t*);
-INSTANTIATE(int32_t, int64_t, int32_t, int8_t, int8_t, int32_t, const int32_t*);
-INSTANTIATE(int32_t, int64_t, int64_t, int8_t, int8_t, int32_t, const int32_t*);
-INSTANTIATE(float, int32_t, int32_t, int8_t, int8_t, float, const float*);
-INSTANTIATE(float, int64_t, int32_t, int8_t, int8_t, float, const float*);
-INSTANTIATE(float, int64_t, int64_t, int8_t, int8_t, float, const float*);
 #undef INSTANTIATE
