@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -38,6 +38,8 @@ rocsparse_status rocsparse::gtsv_buffer_size_template(rocsparse_handle handle,
                                                       rocsparse_int    ldb,
                                                       size_t*          buffer_size)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     ROCSPARSE_CHECKARG_HANDLE(0, handle);
 
     // Logging
@@ -125,6 +127,8 @@ namespace rocsparse
                                                        rocsparse_int    ldb,
                                                        void*            temp_buffer)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         char* ptr    = reinterpret_cast<char*>(temp_buffer);
         T*    dl_pad = reinterpret_cast<T*>(ptr);
         ptr += ((sizeof(T) * m_pad - 1) / 256 + 1) * 256;
@@ -504,6 +508,8 @@ rocsparse_status rocsparse::gtsv_template(rocsparse_handle handle,
                                           rocsparse_int    ldb,
                                           void*            temp_buffer)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     ROCSPARSE_CHECKARG_HANDLE(0, handle);
 
     // Logging
@@ -623,6 +629,7 @@ rocsparse_status rocsparse::gtsv_template(rocsparse_handle handle,
                                      size_t*          buffer_size)                               \
     try                                                                                          \
     {                                                                                            \
+        ROCSPARSE_ROUTINE_TRACE;                                                                 \
         RETURN_IF_ROCSPARSE_ERROR(                                                               \
             rocsparse::gtsv_buffer_size_template(handle, m, n, dl, d, du, B, ldb, buffer_size)); \
         return rocsparse_status_success;                                                         \
@@ -651,6 +658,7 @@ C_IMPL(rocsparse_zgtsv_buffer_size, rocsparse_double_complex);
                                      void*            temp_buffer)                   \
     try                                                                              \
     {                                                                                \
+        ROCSPARSE_ROUTINE_TRACE;                                                     \
         RETURN_IF_ROCSPARSE_ERROR(                                                   \
             rocsparse::gtsv_template(handle, m, n, dl, d, du, B, ldb, temp_buffer)); \
         return rocsparse_status_success;                                             \
